/*
 * rc.h - 2-channel in, 2-channel out RC PWM (900-1100us 50Hz typical)
 * For ATmega48/88/168/328 (and possibly others)
 * Copyright (c) 2015 James Rowley - james@jrowley.me
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 */ 

#include "defines.h"
#include "rc.h"

uint16_t rcIn[ICHAN],rcOut[OCHAN];
//uint16_t usIn[ICHAN],usOut[OCHAN];
uint8_t rcPinsRX[ICHAN] = {RX_PINS};

void initRC(){
	PCICR=RX_PCICR;
	RX_PCMSK |= RXMSK;
	RXDDR &=~ RXMSK;
	RXPORT |= RXMSK;
	TCCR1B |= (1<<CS10);
	TCCR1B |= (1<<CS11);
	TCCR1A |=  (1<<WGM10);
	TCCR1A |=  (1<<WGM11);
	TCCR1A &=~ (1<<WGM12);
	TCCR1A &=~ (1<<WGM13);
	TCCR1A |= (1<<COM1A1);
	TCCR1A |= (1<<COM1B1);
	DDRB |= (1<<PB1);
	DDRB |= (1<<PB2);
}

ISR(RX_PCINT){
	cli();
	uint8_t change, input;
	uint16_t cTime,dt;
	static uint16_t rcTime[ICHAN];
	static uint8_t lastInput;
	input=RXPIN;
	change=input^lastInput;
	cTime=micros();
	lastInput=input;
	sei();
	
	for(int i=0;i<ICHAN;i++){
		if(change&rcPinsRX[i]){
			dt=cTime-rcTime[i];
			if(dt>RCMIN && dt<RCMAX){
				rcIn[i]=dt;
			}else{
				rcTime[i]=cTime;
			}
		}
	}
}

void outputPWM(){
	//OCR1A = rcOut[0]<<4;
	OCR1A = (rcOut[1]/9)-70;
	OCR1B = (rcOut[0]/9)-70;
}

void setPWM( uint8_t chan, uint16_t val ){
	rcOut[chan]=val;
	outputPWM();
}
